/**
 * Created by stefan on 17.03.15.
 */

/*

procs -----------memory---------- ---swap-- -----io---- -system-- ------cpu-----
  r  b   swpd   free   buff  cache   si   so    bi    bo   in   cs us sy id wa st
1  0      0 8007492 337728 2513360    0    0     7    56  152  165  6  2 92  0  0

*/

/*
 * @copyright Copyright (c) Sematext Group, Inc. - All Rights Reserved
 *
 * @licence SPM for NodeJS is free-to-use, proprietary software.
 * THIS IS PROPRIETARY SOURCE CODE OF Sematext Group, Inc. (Sematext)
 * This source code may not be copied, reverse engineered, or altered for any purpose.
 * This source code is to be used exclusively by users and customers of Sematext.
 * Please see the full license (found in LICENSE in this distribution) for details on its license and the licenses of its dependencies.
 */
'use strict'
var SpmAgent = require('spm-agent')
var Agent = SpmAgent.Agent
var config = SpmAgent.Config
var os = require('os')
var cluster = require('cluster')
var exec = require('exec')

/**
 * This module generates OS Metrics. Names are compatible with collectd. SpmSender uses collectd format
 * @returns {Agent}
 */
module.exports = function () {
  var osAgent = new Agent(
    {
      /*
       vmstat -s
       0 12320804 K total memory
       1 1400124 K used memory
       2 671848 K active memory
       3 610676 K inactive memory
       4 10920680 K free memory
       5 68568 K buffer memory
       6 739608 K swap cache
       7 12462076 K total swap
       8 0 K used swap
       9 12462076 K free swap
       10 22129 non-nice user cpu ticks
       11 100 nice user cpu ticks
       12 6102 system cpu ticks
       13 229937 idle cpu ticks
       14 28735 IO-wait cpu ticks
       15 0 IRQ cpu ticks
       16 149 softirq cpu ticks
       17 0 stolen cpu ticks
       18 636827 pages paged in
       19 257508 pages paged out
       20 0 pages swapped in
       21 0 pages swapped out
       22 657102 interrupts
       23 3410066 CPU context switches
       24 1426593249 boot time
       25 3100 forks
       */
      getVmStatMetrics: function (callback) {
           exec ('vmstat', function (err, out, code) {
                if (code == 0)
                {
                    var lines = out.split ('\n')

                    if (lines.length >= 25) {
                      // r  b   swpd   free   buff  cache   si   so    bi    bo   in   cs us sy id wa st
                      var swapMapping = {
                        swapUsed: 7,
                        swapIn: 20,
                        swapOut: 21
                      }
                      var cpuMapping = {
                        nice: 11,
                        sys: 12,
                        idle: 13,
                        wait: 14,
                        irq: 15,
                        softirq: 16,
                        stolen: 17
                      }

                      var mapValues = function (mapping) {
                        var result = {}
                        for (var property in mapping) {
                          result [property] = lines[mapping[property]].split(' ')[0]
                        }
                        return result
                      }
                      var rv = {
                        swap: mapVaules(swapMapping)
                        cpu: mapValues(cpuMapping)
                      }
                      callback(null, rv)
                  }
                } else {
                  callback (new Error ('Error executing vmstat: [' code + ']' + err))
                }
           })
      },
      start: function (agent) {
        var cpuLastValues = {}
        os.cpus().forEach(function (cpu, i) {
          cpuLastValues[i] = {idle: 0, user: 0, sys: 0, irq: 0, nice: 0, wait: 0, softirq: 0, stolen: 0}
        })
        var cpuProperties = ['user', 'nice', 'irq', 'sys', 'idle']
        if (cluster.isMaster) {
          var timerId = setInterval(function () {
            var time = new Date().getTime()
            var metrics = {}
            metrics['collectd-sys-load'] = os.loadavg()
            metrics['collectd-mem-free'] = os.freemem()
            metrics['collectd-mem-used'] = os.totalmem() - os.freemem()
            getVmStatMetrics (function (err, vmstatMetrics) {
              if (err) {
                logger.error(err)
                return
              }
              var cpuTotal = 0
              vmstatMetrics.cpu.forEach(function (property) {
                cpuLastValues[i][property] = cpu.times[property] - cpuLastValues[i][property]
                cpuTotal = cpuTotal + cpuLastValues[i][property]
              })
              metrics['collectd-cpu-usr\tcpu-' + i + '/cpu-user'] = (cpuLastValues[i].user / cpuTotal) * 100
              metrics['collectd-cpu-nic\tcpu-' + i + '/cpu-nice'] = (cpuLastValues[i].nice / cpuTotal) * 100
              metrics['collectd-cpu-sys\tcpu-' + i + '/cpu-system'] = (cpuLastValues[i].sys / cpuTotal) * 100
              metrics['collectd-cpu-idl\tcpu-' + i + '/cpu-idle'] = (cpuLastValues[i].idle / cpuTotal) * 100
              metrics['collectd-cpu-int\tcpu-' + i + '/cpu-interrupt'] = (cpuLastValues[i].irq / cpuTotal) * 100
              metrics['collectd-cpu-ste\tcpu-' + i + '/cpu-steal'] = (cpuLastValues[i].stolen / cpuTotal) * 100
              metrics['collectd-cpu-sir\tcpu-' + i + '/cpu-softirq'] = (cpuLastValues[i].softirq / cpuTotal) * 100
              metrics['collectd-cpu-wait\tcpu-' + i + '/cpu-wait'] = (cpuLastValues[i].wait / cpuTotal) * 100
            })
            // setting "sct" parameter for spm-sender to use specific collectd format when transmitting to receiver.
            for (var x in metrics)
              agent.addMetrics({ts: time, name: x, value: metrics[x], sct: 'OS'})

            agent.addMetrics({
              ts: time,
              name: 'numWorkers',
              value: Object.keys(cluster.workers).length
            })

          }, config.collectionInterval || 30000)
          if (timerId.unref)
            timerId.unref()
        } else {
          // osmetrics deactivated on worker processes
        }
      }
    }
  )
  return osAgent
}

